﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs
{
	using System;
	using dbfs.model;
	using dbfs.repository;
	using dbfs.utils;

	public interface ISource
	{
	}

	public interface IFileSource : ISource
	{
	}

	public interface IDirectorySource : ISource
	{
	}

	internal class Source : ISource
	{
		public File File { get; set; }
		public Folder Folder { get; set; }
		public string Path { get; set; }
		public int BranchID { get; set; }
		public int CommitID { get; set; }

		public string Name()
		{
			if (File != null) return File.Name;
			if (Folder != null) return Folder.Name;
			if (Path != null) return FileUtils.GetFileName(Path);

			throw new InvalidOperationException();
		}

		public string GetPath()
		{
			if (File != null) return File.Path;
			if (Folder != null) return Folder.Path;

			return Path;
		}

		public Info GetInfo(Repository db)
		{
			FileRevision revision = null;

			if (File == null)
				revision = db.Revision.GetOptimized(db.Branch.Get(BranchID), Path, CommitID);

			return new Info {
				RevisionID = File != null ? File.RevisionID : revision.id,
				Blob = File != null ? db.Blob.TryGet(File.BlobID) : revision._(e => e.Blob)
			};
		}


		public class Info
		{
			public int RevisionID { get; set; }
			public Blob Blob { get; set; }
		}
	}

	internal class FileSource : Source, IFileSource
	{
	}

	internal class DirectorySource : Source, IDirectorySource
	{
	}
}
