﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.repository
{
	using System.Collections.Generic;
	using System.Linq;
	using dbfs.model;

	static class Extensions
	{
		public static IQueryable<FileRevision> SkipRemoved(this IQueryable<FileRevision> self)
		{
			return self.Where(e => e.type != (int) ChangeType.DELETE && e.type != (int) ChangeType.RENAMED);
		}

		public static IQueryable<LatestRevision> SkipRemoved(this IQueryable<LatestRevision> self)
		{
			return self
				.Where(e => e.FileRevision.type != (int) ChangeType.DELETE
					&& e.FileRevision.type != (int) ChangeType.RENAMED);
		}

		public static bool IsRemoved(this FileRevision self)
		{
			return self.type == (int) ChangeType.DELETE || self.type == (int) ChangeType.RENAMED;
		}

		public static IEnumerable<dbfs.Commit> Wrap(this IEnumerable<Commit> self, Branch branch)
		{
			var branch_ = new dbfs.Branch(branch);

			return self.Select(e => new dbfs.Commit(e, branch_));
		}
	}
}
