﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.repository
{
	using System.Collections.Generic;
	using System.Linq;
	using dbfs.model;
	using dbfs.utils;

	class LatestManagement : RepositoryManagement
	{
		/// <summary>
		/// Optimization: Attaches LatestRevision to the branch instead of DbSet.
		/// </summary>
		public LatestRevision Create(Branch branch, FileRevision revision)
		{
			var latest = new LatestRevision {
				Branch = branch,
				File = revision.File,
				FileRevision = revision,
			};

			branch.LatestRevisions.Add(latest);

			return latest;
		}

		#region getters

		public LatestRevision TryGet(int branch_id, string path, bool skip_removed = false)
		{
			return _context.LatestRevisions
				.Where(e => e.branch_id == branch_id)
				.Where(e => e.File.path == path)
				.If(skip_removed, q => q.SkipRemoved())
				.FirstOrDefault()
			;
		}

		/// <param name="only_folders">true - only folders, false - only files, null - folders and files</param>
		public List<LatestRevision> List(int branch_id, string directory,
			bool? only_folders = null, bool skip_removed = false)
		{
			return _context.LatestRevisions
				.Where(e => e.branch_id == branch_id)
				.Where(e => e.File.folder == directory)
				.If(only_folders.HasValue, q => q.Where(e => e.File.is_directory == only_folders.Value))
				.If(skip_removed, q => q.SkipRemoved())
				.ToList()
			;
		}

		public List<LatestRevision> AllNested(int branch_id, string directory, bool skip_removed = false)
		{
			return _context.LatestRevisions
				.Where(e => e.branch_id == branch_id)
				.Where(e => e.File.path.StartsWith(directory))
				.If(skip_removed, q => q.SkipRemoved())
				.ToList()
			;
		}

		#endregion
	}
}
