﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Data.Entity.Validation;
using dbfs.utils;
using dbfs.model;

namespace dbfs.repository
{
	class Repository : IDisposable
	{
		public Context _context;
		private string _config;

		public Repository(string configName = "dbfs")
		{
			_config = configName;
			_context = Context.Create(_config);

			Commit = create<CommitManagement>();
			Branch = create<BranchManagement>();
			File = create<FileManagement>();
			Latest = create<LatestManagement>();
			Revision = create<RevisionManagement>();
			Blob = create<BlobManagement>();
			User = create<UserManagement>();
        }

		public CommitManagement Commit { get; private set; }
		public BranchManagement Branch { get; private set; }
		public FileManagement File { get; private set; }
		public LatestManagement Latest { get; private set; }
		public RevisionManagement Revision { get; private set; }
		public BlobManagement Blob { get; private set; }
		public UserManagement User { get; private set; }

		T create<T>() where T : RepositoryManagement, new()
		{
			T result = new T();
			result.Repository = this;
			return result;
		}

		public void Reload()
		{
			_context.Dispose();
			_context = Context.Create(_config);
		}

        public void Dispose()
        {
            _context.Dispose();
        }

        public void SaveChanges()
        {
			try
			{
				_context.SaveChanges();
			}
			catch (DbEntityValidationException e)
			{
				foreach (var eve in e.EntityValidationErrors)
				{
					foreach (var ve in eve.ValidationErrors)
					{
						Log.Error("{0}.{1}: {2}", eve.Entry.Entity.GetType().Name, ve.PropertyName, ve.ErrorMessage);
					}
				}

				throw;
			}
		}
    }
}
