/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace dbfs.dbfscmd
{
	using System;
	using System.IO;
	using dbfs;
	using dbfs.utils;

	public static class CreateCommitExtensions
	{
		public static void AddFilesFromDirectory(this CreateCommit commit, string directory, string prefix, bool recursive)
		{
			foreach (var filename in Directory.EnumerateFiles(directory)) {
				var virtual_path = FileUtils.CombinePath(prefix, FileUtils.GetFileName(filename));

				commit.AddFile(virtual_path, GetFileContentLoader(filename));
			}

			if (recursive) {
				foreach (var directory_name in Directory.EnumerateDirectories(directory)) {
					string next_prefix = FileUtils.CombinePath(prefix, FileUtils.GetFileName(directory_name));

					AddFilesFromDirectory(commit, directory_name, next_prefix, true);
				}
			}
		}

		private static Func<string, Stream> GetFileContentLoader(string filename)
		{
			return path => new FileStream(filename, FileMode.Open);
		}
	}
}
