﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Collections;
using System.Dynamic;
using System.Linq;
using System.Reflection;

namespace dbfs.utils
{
	public class ComWrapper : DynamicObject
	{
		public ComWrapper(object comObject)
		{
			_comObject = comObject;
			_type = _comObject.GetType();
		}

		public object WrappedObject { get { return _comObject; } }

		public override bool TryGetMember(GetMemberBinder binder, out object result)
		{
			result = Wrap(_type.InvokeMember(
				binder.Name, BindingFlags.GetProperty, null, _comObject, null
			));
			return true;
		}

		public override bool TrySetMember(SetMemberBinder binder, object value)
		{
			_type.InvokeMember(
				binder.Name, BindingFlags.SetProperty, null, _comObject,
				new object[] { Unwrap(value) }
			);
			return true;
		}

		public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result)
		{
			result = Wrap(_type.InvokeMember(
				binder.Name, BindingFlags.InvokeMethod, null, _comObject,
				args.Select(arg => Unwrap(arg)).ToArray()
			));
			return true;
		}

		public override bool TryConvert(ConvertBinder binder, out object result)
		{
			if (binder.Type.Equals(typeof(IEnumerable)) && _comObject is IEnumerable)
			{
				result = Enumerate();
				return true;
			}

			result = null;
			return false;
		}

		public override bool TryGetIndex(GetIndexBinder binder, object[] indexes, out object result)
		{
			if (indexes.Length == 1)
			{
				dynamic indexer = _comObject;
				result = Wrap(indexer[indexes[0]]);
				return true;
			}

			result = null;
			return false;
		}

		public override bool TrySetIndex(SetIndexBinder binder, object[] indexes, object value)
		{
			if (indexes.Length == 1)
			{
				dynamic indexer = _comObject;
				indexer[indexes[0]] = Unwrap(value);
				return true;
			}

			return false;
		}

		private object Wrap(object obj)
		{
			return obj != null && obj.GetType().IsCOMObject
				? new ComWrapper(obj)
				: obj;
		}

		private object Unwrap(object obj)
		{
			ComWrapper wrapper = obj as ComWrapper;
			return wrapper != null ? wrapper._comObject : obj;
		}

		private IEnumerable Enumerate()
		{
			foreach (var item in (IEnumerable)_comObject)
				yield return Wrap(item);
		}

		private object _comObject;
		private Type _type;
	}
}
