﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.IO;

namespace dbfs.utils
{
	public class StreamWithEvents : Stream
	{
		public delegate void StreamEventHandler(byte[] buffer, int offset, int count);

		public StreamWithEvents(Stream stream, StreamEventHandler onRead, StreamEventHandler onWrite)
		{
			if (stream == null)
				throw new ArgumentNullException("stream");

			_inner = stream;
			_onread = onRead;
			_onwrite = onWrite;
		}

		public override bool CanSeek { get { return _inner.CanSeek; } }
		public override bool CanRead { get { return _inner.CanRead; } }
		public override bool CanWrite { get { return _inner.CanWrite; } }
		public override long Length { get { return _inner.Length; } }

		public override void SetLength(long value)
		{
			_inner.SetLength(value);
		}

		public override long Position
		{
			get { return _inner.Position; }
			set { _inner.Position = value; }
		}

		public override long Seek(long offset, SeekOrigin origin)
		{
			return _inner.Seek(offset, origin);
		}

		public override int Read(byte[] buffer, int offset, int count)
		{
			int result = _inner.Read(buffer, offset, count);

			if (_onread != null)
				_onread(buffer, offset, result);

			return result;
		}

		public override void Write(byte[] buffer, int offset, int count)
		{
			if (_onwrite != null)
				_onwrite(buffer, offset, count);

			_inner.Write(buffer, offset, count);
		}

		public override void Flush()
		{
			_inner.Flush();
		}

		protected Stream _inner;
		protected StreamEventHandler _onread;
		protected StreamEventHandler _onwrite;
	}
}
