﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System.Data;
using System.Data.Common;
using System.Data.Objects.DataClasses;
using System.Data.EntityClient;
using System.Data.SqlClient;
using System.IO;
using dbfs.utils.db.impl;
using dbfs.utils.db.impl.mssql;

namespace dbfs.utils.db
{
	public static class DatabaseUtils
	{
		public static IBinaryFieldReader GetFieldReader(this DbConnection connection, string schema, string table, string read_field, string key_field, object key)
		{
			return new BinaryFieldReader(GetDirectConnection(connection), schema, table, read_field, key_field, key);
		}

		public static Stream GetFieldReaderStream(this DbConnection connection, string schema, string table, string read_field, string key_field, object key)
		{
			return new FieldReaderStream(GetFieldReader(connection, schema, table, read_field, key_field, key));
		}

		public static IBinaryFieldReader GetFieldReader(this DbConnection connection, EntityReference reference, string field)
		{
			EntityKeyMember key = reference.EntityKey.EntityKeyValues[0];

			// fixme:
			string schema = null;
			// entity set may have pluralized name of the real table name
			string table_name = reference.TargetRoleName; //reference.EntityKey.EntitySetName

			return GetFieldReader(connection, schema, table_name, field, key.Key, key.Value);
		}

		public static Stream GetFieldReaderStream(this DbConnection connection, EntityReference reference, string field)
		{
			return new FieldReaderStream(GetFieldReader(connection, reference, field));
		}

		public static IBinaryFieldWriter GetFieldWriter(this DbConnection connection, string schema, string table, string write_field, string key_field, object key)
		{
			connection = GetDirectConnection(connection);

			if (connection is SqlConnection)
				return new MsSqlBinaryFieldWriter(connection, schema, table, write_field, key_field, key);
				//return new MsSqlImageFieldWriter(connection, table, write_field, key_field, key);

			return new BinaryFieldWriter(connection, schema, table, write_field, key_field, key);
		}

		public static Stream GetFieldWriterStream(this DbConnection connection, string schema, string table, string write_field, string key_field, object key)
		{
			return new FieldWriterStream(GetFieldWriter(connection, schema, table, write_field, key_field, key));
		}

		public static IBinaryFieldWriter GetFieldWriter(this DbConnection connection, EntityReference reference, string field)
		{
			EntityKeyMember key = reference.EntityKey.EntityKeyValues[0];

			// fixme:
			string schema = null;
			// entity set may have pluralized name of the real table name
			string table_name = reference.TargetRoleName; //reference.EntityKey.EntitySetName

			return GetFieldWriter(connection, schema, table_name, field, key.Key, key.Value);
		}

		public static Stream GetFieldWriterStream(this DbConnection connection, EntityReference reference, string field)
		{
			return new FieldWriterStream(GetFieldWriter(connection, reference, field));
		}

		public static void SetFieldFromStream(this DbConnection connection, Stream stream, string schema, string table, string write_field, string key_field, object key)
		{
			using (var output = GetFieldWriterStream(connection, schema, table, write_field, key_field, key))
            {
				stream.SaveToStream(output);
			}
		}

		public static void SetFieldFromStream(this DbConnection connection, Stream stream, EntityReference reference, string field)
		{
			EntityKeyMember key = reference.EntityKey.EntityKeyValues[0];

			// fixme:
			string schema = null;
			// entity set may have pluralized name of the real table name
			string table_name = reference.TargetRoleName; //reference.EntityKey.EntitySetName

			SetFieldFromStream(connection, stream, schema, table_name, field, key.Key, key.Value);
		}

		public static void SetFieldFromFile(this DbConnection connection, string filename, EntityReference reference, string field)
		{
			using (var file = File.OpenRead(filename))
			{
				SetFieldFromStream(connection, file, reference, field);
			}
		}

		private static DbConnection GetDirectConnection(DbConnection connection)
		{
			EntityConnection conn = connection as EntityConnection;

			if (conn != null)
				connection = conn.StoreConnection;

			if (connection.State != ConnectionState.Open)
				connection.Open();

			return connection;
		}
	}
}
