﻿/*
	DbFS.NET
    Copyright (C) 2016 SCAND Ltd, svg@scand.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Data;
using System.Data.Common;

namespace dbfs.utils.db.impl
{
	class BinaryFieldReader : IBinaryFieldReader
	{
		public BinaryFieldReader(DbConnection connection, string schema, string table, string read_field, string key_field, object key)
		{
			_cmd = connection.CreateCommand();
			_cmd.CommandTimeout = 5 * 60;

			_cmd.CommandText = string.Format(
				"SELECT [{0}] FROM {1} WHERE [{2}]=@KeyValue",
				read_field, Helpers.TableSource(schema, table), key_field
			);

			DbParameter key_value = _cmd.CreateParameter();
			key_value.ParameterName = "@KeyValue";
			key_value.Value = key;
			_cmd.Parameters.Add(key_value);

			_reader = _cmd.ExecuteReader(CommandBehavior.SequentialAccess);
			_reader.Read();

			Size = (int)_reader.GetBytes(0, 0, null, 0, 0);
		}

		public virtual int Size { get; private set; }
		public virtual int Position { get; set; }

		public virtual int Read(byte[] buffer, int offset, int count)
		{
			try
			{
				int returned = (int)_reader.GetBytes(0, Position, buffer, offset, count);

				if (returned > 0)
					Position += returned;

				return returned;
			}
			catch (IndexOutOfRangeException)
			{
				return 0;
			}
		}

		public virtual void Dispose()
		{
			if (_reader != null)
			{
				_reader.Dispose();
				_reader = null;
			}

			if (_cmd != null)
			{
				_cmd.Dispose();
				_cmd = null;
			}
		}

		private DbCommand _cmd;
		private DbDataReader _reader;
	}
}
